/*
 * StateFunction.java
 *
 * Created on 12. nor 2006, 22:39
 *
 */

package savi.alg.funcmin;

import savi.alg.StateSkeleton;
import java.util.*;
import savi.alg.knapsack.StateKnapsack;

/**
 * State representing finding global minimum problem.
 * @author Premysl JIRIK
 */
public class StateFunction implements StateSkeleton {
    /**
     * current value x in graph
     */
    protected double currentX;
    /**
     * half of pixel size
     */
    protected static double half = 210;
    
    /** Creates a new instance of StateFunction */
    public StateFunction() {
    }

    /**
     * implemented function
     * @param x value x of function
     * @return value y
     */
    public double fce(double x) {
        x /= 3;
        return (20 * Math.sin(0.3 * x) * Math.sin(1.3 * x) - 0.04 * (x * x)) + 
                (0.00002 * (x * x * x * x) + 0.4 * x + 160);
    }    
    /**
     * Initializes state.
     * @param temp Starting temperature
     */
    public void findInitialState(double temp) {
        currentX = -half + (int)(2 * half * Math.random());
    }
    /**
     * Called when algorithm need to find next state.
     * @param temp actual temperature
     */
    public void findNextState(double temp) {
        currentX = -half + (int)(2 * half * Math.random());
    }
    /**
     * Count optimalization criterion.
     * @return optimalization criterion for the state
     */
    public double getActualCost() {
        return fce(currentX);
    }
    /**
     * Used for taking current x value (that means state of function).
     * @return x coordinate
     */
    public double getCurrentX() {
        return currentX;
    }
    /**
     * Create new copy of StateFunction object.
     * @return deep copy of StateFunction
     */
    public Object clone() {
        StateFunction copy = null;
        try {
            copy = (StateFunction)super.clone();
        } 
        catch (CloneNotSupportedException ex) {
            ex.printStackTrace();
        }
        return copy;
    }    
    
    /**
     * Not implemented, Finding global minimum problem doesn't allow loading and saving
     * submissions.
     * @return null
     */
    public String [] getSubmission() {
        // should not be called! :)
        return null;
    } 
}
