/*
 * ViewKnapsack.java
 *
 * Created on 12. nor 2006, 15:52
 *
 */

package savi.alg.knapsack;

import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.text.DecimalFormat;
import savi.alg.*;
import savi.visualization.support.VisualizationPanel;

/**
 * Visualization panel class for knapsack problem.
 * @author Premysl JIRIK
 */
public class ViewKnapsack extends VisualizationPanel {
    /**
     * reference to list of items
     */
    protected Items                   itmList = null;
    /**
     * scrollable panel for list
     */
    protected ScrollPane              pane;
    /**
     * drawer for list of items
     */
    protected KnapsackListCanvas      draw;
    
    /**
     * Creates a new instance of ViewKnapsack.
     */
    public ViewKnapsack() {
        super();
        initScreen();
    }
    /**
     * Creates a new instance of ViewKnapsack. Initializes internal reference to state.
     * @param s state
     */
    public ViewKnapsack(StateSkeleton s) {
        setState(s);
        initScreen();
    }
    
    /**
     * Initialize draw of items.
     */
    protected void initScreen() {
        setBackground(Color.WHITE);
        
        pane = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);
        pane.setBounds(5, 5, 410, 435);
        add(pane);
        pane.setWheelScrollingEnabled(true);
        pane.getVAdjustable().setUnitIncrement(16);
        
        
        draw = new KnapsackListCanvas(this);
        draw.setSize(new Dimension(200, 200));
        pane.add(draw);
        
        validate();
        setVisible(true);

/*        this.add(lstItems = new List(1, true));
        lstItems.setBounds(5, 5, 410, 435);
        lstItems.setBackground(new Color(230, 230, 230));
        lstItems.setVisible(true);
        
        lstItems.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                int idx = (Integer)(e.getItem());
                if (e.getStateChange() == ItemEvent.DESELECTED) 
                    lstItems.select(idx);
                else
                    lstItems.deselect(idx);
            }
        });
 */
    }
    
    /**
     * Updates scene
     */
    public void updateScene() {
        if (itmList != ((StateKnapsack)state).getItems())
            itmList = ((StateKnapsack)state).getItems();
        draw.refresh();
    }

    class KnapsackListCanvas extends Canvas {
        int                 imageHeight = -1;
        Image               buffImage;
        Graphics            offscreen;
        boolean             initDone = false;

        KnapsackListCanvas(VisualizationPanel vp) {
            super();
            setBackground(Color.WHITE);
        }
        
        public void paint(Graphics g) {
            if (!initDone)
                initpaint(g);
            g.drawImage(buffImage, 0, 0, this);
            pane.validate();
        }

        public void update(Graphics g) {
            if (!initDone)
                initpaint(g);
            g.drawImage(buffImage, 0, 0, this);
        }
        
        public void refresh() {
            if ((itmList.count * 16 + 10) != draw.getHeight()) {
                this.setSize(300, itmList.count * 16 + 10);
                this.validate();
            }
            initDone = false;
            repaint();
        }
        
        protected void initpaint(Graphics g) {
            int desiredHeight = itmList.count * 16 + 10;
            try {
                if (imageHeight != desiredHeight) {
                    buffImage = this.createImage(300, desiredHeight);
                    offscreen = buffImage.getGraphics();
                    imageHeight = desiredHeight;
                }
                offscreen.setColor(Color.WHITE);
                offscreen.fillRect(0, 0, 300, desiredHeight);
                
                boolean [] sack = ((StateKnapsack)state).getKnapsack();
                String fmtNum = "%04d";
                String fmtIdx = "%03d";
            
                for (int i = 0; i < itmList.count; i++) {
                    offscreen.setColor(Color.WHITE);
                    offscreen.fillRect(0, i * 16, 300, (i + 1) * 16);
                    if (sack[i]) {
                        offscreen.setColor(Color.GREEN);
                        offscreen.fillRect(0, i * 16 + 1, 300, (i + 1) * 16 - 1);
                    }
                    offscreen.setColor(Color.BLACK);
                    offscreen.drawString("Item #" + String.format(fmtIdx, i), 5, i * 16 + 13);
                    offscreen.drawString("Weight " + Integer.toString((int)itmList.getItemWeight(i)), 100, i * 16 + 13);
                    offscreen.drawString("Cost " + Integer.toString((int)itmList.getItemCost(i)), 200, i * 16 + 13);
                }
                initDone = true;
            }
            catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}

/*
            lstItems.removeAll();
 
            DecimalFormat idx = new DecimalFormat("0");
            DecimalFormat num = new DecimalFormat("0");
                lstItems.add("Item #" + idx.format(i) + " - Weight: " 
                        + num.format(itmList.getItemWeight(i)) + "     Cost: "
                        + num.format(itmList.getItemWeight(i)));

            String fmtNum = "%04d";
            String fmtIdx = "%03d";
            for (int i = 0; i < itmList.getCount(); i++)
                lstItems.add("Item #" + String.format(fmtIdx, i) + " - Weight: " 
                        + String.format(fmtNum, (int)itmList.getItemWeight(i)) + "     Cost: "
                        + String.format(fmtNum, (int)itmList.getItemCost(i)));
        }
        boolean []sack = ((StateKnapsack)alg.getState()).getKnapsack();
        for (int i = 0; i < itmList.getCount(); i++) {
            if (sack[i]) 
                lstItems.select(i);
            else
                lstItems.deselect(i);
        }
        lstItems.makeVisible(0);
*/

/*
    
class myCanvas extends Canvas {
  Image    buffImage;
  Graphics offscreen;
  boolean initDone = false;

  myCanvas() { super(); }
    public void paint(Graphics g) {
   if (!initDone)
      initpaint(g);
   else
      g.drawImage(buffImage, 0, 0,  this);
   }

  public void update(Graphics g) {
   g.drawImage(buffImage, 0, 0,  this);
   }

  public void initpaint(Graphics g) {
   try {
    buffImage = this.createImage(500, 500);
    offscreen = buffImage.getGraphics();
    offscreen.setColor(Color.black);
    offscreen.fillRect(0, 0, 500, 500);
    offscreen.setColor(Color.white);
    offscreen.setFont(new Font("Courier", Font.ITALIC, 42));
    offscreen.drawString("Hello World!", 0, 50);
    initDone = true;
    g.drawImage(buffImage,0,0, this);
    }
   catch (Exception e) {
    e.printStackTrace();
    }
   }
  }
*/ 