/*
 * ViewTSP.java
 *
 * Created on 11. nor 2006, 11:14
 *
 */

package savi.alg.tsp;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.Vector;
import savi.alg.*;
import savi.alg.SimAnnealingSkeleton;
import savi.visualization.support.SimPanel;
import savi.visualization.support.VisualizationPanel;

/**
 * Visualization panel for TSP (traveling salesman problem).
 * @author Premysl JIRIK
 */
public class ViewTSP extends VisualizationPanel implements MouseListener {
    boolean editMode = false;
    Vector clickList = new Vector();
    
    /** Creates a new instance of ViewTSP */
    public ViewTSP() {
        setBackground(Color.WHITE);
        addMouseListener(this);
    }
    /**
     * Creates a new instance of ViewTSP, initialize state.
     * @param s state
     */
    public ViewTSP(StateSkeleton s) {
        setState(s);
        setBackground(Color.WHITE);
    }
    
    /**
     * Updates scene (repaint visualization)
     */
    public void updateScene() {
        repaint();
    }
    
    /**
     * Paints this canvas.
     * @param g the graphics context to use for painting
     */
    public void paint(Graphics g) {
        super.paint(g);
        
        int w = getWidth() - 10;
        int h = getHeight() - 10;
        
        if (editMode) {
            // only red dots - editing
            g.setColor(Color.RED);
            for (int i = 0; i < clickList.size(); i++) {
                double [] d = (double [])clickList.elementAt(i);
                g.fillOval(5 + (int)(d[0] / 20.0 * w) - 2, 5 + (int)(d[1] / 20.0 * h) - 2, 4, 4);
            }
            g.setColor(Color.ORANGE);
            g.drawString("Use left mouse button to add city", 10, 15);
            g.drawString("Use right mouse button confirm", 10, 30);
        } else {
            // standard view
            StateTSP s = (StateTSP)state;
            Cities c = s.getCities();
            int len = c.getCount();

            g.setColor(Color.BLACK);
            for (int i = 0; i < len; i++) {     // draw routes
                int a = s.path[i];
                int b = s.path[(i + 1) % len];

                g.drawLine(5 + (int)(c.getCityX(a) * w), 5 + (int)(c.getCityY(a) * h), 
                        5 + (int)(c.getCityX(b) * w), 5 + (int)(c.getCityY(b) * h));
            }     

            g.setColor(Color.BLUE);
            for (int i = 0; i < len; i++) {
                g.fillOval(5 + (int)(c.getCityX(i) * w) - 2, 5 + (int)(c.getCityY(i) * h) - 2, 4, 4);
            }
        }
    }
    
    
    /**
     * Switch visualizazion panel to state when user can edit cities
     */
    public void startEditMode() {
        editMode = true;
/* hack */
        ((SimPanel)this.getParent()).setViewCurrent(true);
        ((SimPanel)this.getParent()).setEnableIDE(false);
        ((SimPanel)this.getParent()).activateView(1);
/* /hack */        
        clickList.clear();
    }
    /**
     * Switch to standard mode, return inserted cities.
     * @return list of cities
     */
    public Vector stopEditMode() {
        editMode = false;
/* hack */
        ((SimPanel)this.getParent()).setEnableIDE(true);
/* /hack */        
        return clickList;
    }

    /**
     * Handles the mouseClicked event, used for cities inserting.
     * @param e mouse event
     */
    public void mouseClicked(MouseEvent e) { 
        if (e.getButton() == MouseEvent.BUTTON1) {
            double d[] = new double[2];
            d[0] = (double)(e.getX() - 5) / (double)(getWidth() - 10);
            d[1] = (double)(e.getY() - 5) / (double)(getHeight() - 10);
            if ((d[0] >= 0) && (d[0] <= 1) && (d[1] >= 0) && (d[1] <= 1)) {
                d[0] *= 20; d[1] *= 20;
                clickList.add(d);
                repaint();
            }
        }
    }
    /**
     * Not used
     * @param e mouse event
     */
    public void mouseEntered(MouseEvent e) { }
    /**
     * Not used
     * @param e mouse event
     */
    public void mouseExited(MouseEvent e) { }
    /**
     * Not used
     * @param e mouse event
     */
    public void mousePressed(MouseEvent e) { }
    /**
     * Not used
     * @param e mouse event
     */
    public void mouseReleased(MouseEvent e) { }
}

