/*
 * SaviDialog.java
 *
 * Created on 19. erven 2006, 17:56
 *
 */

package savi.dialogs;

import java.awt.*;
import java.awt.event.*;

/**
 * Class for making dialogs.
 * @author Premysl JIRIK
 */
public class SaviDialog extends SaviDialogSkeleton implements ActionListener {
    boolean okPressed = false;
    Button ok, can;
    Panel pan = null;
    int inputRows = 0;
    
    /**
     * Creates a new instance of SaviDialog, initialize dialog caption.
     * @param caption dialog caption
     */
    public SaviDialog(String caption) {
        super(caption);
        
        setLayout(new FlowLayout());
        pan = new Panel();
        pan.setLayout(new GridLayout(2, 2, 10, 3));
        pan.add(new Container());
        pan.add(new Container());
        pan.add(ok = new Button("OK"));
        ok.addActionListener(this); 
        pan.add(can = new Button("Cancel"));
        can.addActionListener(this);
        add(pan);
        
        inputRows = 2;
    }
    
    /**
     * Adds new input with caption.
     * @param caption caption for input
     * @param comp component for input
     */
    public void addInputRow(String caption, Component comp) {
        pan.add(comp, 2 * inputRows - 4);
        pan.add(new Label(caption), 2 * inputRows - 4);
        inputRows++;
    }
    
    /**
     * Return <CODE>true</CODE> if user properly closed dialog (press OK).
     * @return <CODE>true</CODE> if user properly closed dialog (press OK).
     */
    public boolean isOK() {
        return okPressed;
    }
    
    /**
     * Show dialog to user.
     * @return <CODE>true</CODE> if user properly closed dialog (press OK).
     */
    public boolean showForm() {
        distributeKeyListeners();
        ((GridLayout)pan.getLayout()).setRows(inputRows);
        validate();
        pack();
        Dimension d = getToolkit().getScreenSize();
        setLocation((d.width - getWidth())/2, (d.height - getHeight())/2);

        setVisible(true);
        return okPressed;
    }
    
    /**
     * Actions if user press "close".
     */
    protected void closePressed() {
        okPressed = false;
        super.closeCalled();
    }
    
    /**
     * Handles the actionPerformed event - "close" or "ok".
     * @param ae the action event
     */
    public void actionPerformed(ActionEvent ae) {
        if(ae.getSource() == ok) {
            okPressed = true;
            setVisible(false);
        } else 
            if(ae.getSource() == can) {
                okPressed = false;
                setVisible(false);
            }
    }    
}
