/*
 * WeightedSAT3.java
 *
 * Created on 4. z 2006, 9:40
 *
 */

package savi.visualization.addons;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.*;
import java.awt.*;
import java.net.*;
import java.text.DecimalFormat;
import java.io.*;
import savi.alg.StateSkeleton;

import savi.dialogs.*;
import savi.alg.sat3.*;
import savi.alg.maxsat3.*;
import savi.visualization.support.VisualizationInfo;
import savi.visualization.support.SimPanel;

/**
 * Visualization add-on for MAX-3-SAT problem.
 * @author Premysl JIRIK
 */
public class MaxSAT3 extends SimPanel {
    Formula                 formula;
    Label                   lbSatisfy, lbCntSatisfact, lbCntClauses;

    /** Creates a new instance of Knapsack */
    public MaxSAT3() {
        super(new ViewMaxSAT3(), 1, 0.001, 0.99, 100);
    }
    
    /**
     * Initialize problem state.
     */
    protected void initState() {
        state = new StateMaxSAT3(formula);
    }
    
    /**
     * Prepare and return help panel for this problem.
     * @return help panel
     */
    protected VisualizationInfo getInfoPanel() {
        VisualizationInfo vi = new VisualizationInfo();
        vi.addTab("Description", "/res/help/maxsat/maxsat01.gif");
        vi.addTab("SAT -> SAT3", "/res/help/maxsat/maxsat02.gif");
        vi.addTab("Controls", "/res/help/maxsat/maxsat03.gif");
        vi.addTab("Visualization", "/res/help/maxsat/maxsat04.gif");
        vi.addTab("CNF file format", "/res/help/maxsat/maxsat05.gif");
        vi.addTab("CNF example", "/res/help/maxsat/maxsat06.gif");        
        return vi;
    }

    /**
     * Return name of the problem.
     * @return name of the problem
     */
    public String getSimulationName() {
        return "Maximum 3-satisfiability problem (MaxSAT3)";
    }
    
    /**
     * Initialize components in the add-on.
     */
    protected void initComponents() {
        super.initComponents();
        
        addInputButton("Reduce formula").addActionListener(new BrReduceFormula());
        addInputButton("Random form.").addActionListener(new BtRandomItems());
        addInputButton("Save instance").addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                callSaveInstance();
            }
        });
        addInputButton("Load from URL").addActionListener(new btUrlLoad());
        addInputButton("Load from file").addActionListener(new BtFileChoose());
        
        lbSatisfy = addOutputField("Satisfied", "");
        lbCntClauses = addOutputField("Clauses count", "");
        lbCntSatisfact = addOutputField("Satisfied cl.", "");
        
        formula = new Formula(91, 21);
        
        settings.doLayout();
    }   
    
    /**
     * Update info panel (display) with given state.
     * @param s state
     */
    protected void updateDisplay(StateSkeleton s) {
        super.updateDisplay(s);
        
        if (((StateMaxSAT3)s).isSatisfy())
            lbSatisfy.setText("YES");
        else
            lbSatisfy.setText("NO");
        lbCntSatisfact.setText(Integer.toString(((StateMaxSAT3)s).countSatisfactClauses()));
        lbCntClauses.setText(Integer.toString(((StateMaxSAT3)s).getFormula().clausesCount()));
    }
    
/*    protected void initAlg() {
        try {
            ((StateKnapsack)state).setCapacity(Integer.parseInt(tfCapacity.getText()));
        } 
        catch (Exception ex) { }
        
        super.initAlg();
    }*/

    /**
     * Load instance of the problem from stream with bufferedreader.
     * @param br stream with bufferedreader
     */
    protected void loadFromStream(BufferedReader br) {
        try {
            Formula f;
            f = new Formula(br);
            formula = f;
            state = new StateMaxSAT3(f);
            initAlg();
            updateDisplay();
        } catch (WrongInputFormulaData ex) {
            new SaviErrorMessage("WrongInputFormulaData: submission contains syntax error.");
        } catch (Exception ex) {
            new SaviErrorMessage(ex.toString());
        }
    }

    
    class BtFileChoose implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            running(false);
            FileDialog fd = new FileDialog(new Frame(""), "MaxSAT submission", FileDialog.LOAD);
            fd.setVisible(true);

            String curFile;
            if ((curFile = fd.getFile()) != null) {
                String filename = fd.getDirectory() + curFile;
                File f = new File(filename);
                
                try {
                    loadFromStream(new BufferedReader(
                            new InputStreamReader( 
                                new FileInputStream(f))));
                } catch (FileNotFoundException ex) {
                    new SaviErrorMessage("File Not Found: " + filename);
                }
            }
        }
    }
    
    class btUrlLoad implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            running(false);
            SaviUrlDialog sud = new SaviUrlDialog("MaxSAT - load submission from URL");
            if (sud.showForm()) {
                try {
                    URL url = new URL(sud.getURL());
                    URLConnection urlc = (URLConnection)url.openConnection();
                    String userAgent = "Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:1.8) Gecko/20051111 Firefox/1.5";
                    urlc.setRequestProperty("User-Agent", userAgent );      // Identify as Firefox 1.5 ;)
                    urlc.setRequestProperty( "Connection", "close" );       // optionally turn off keep-alive
                    urlc.setAllowUserInteraction(false);
                    urlc.setDoInput(true);
                    urlc.setDoOutput(false);
                    urlc.setUseCaches(false);
                    urlc.connect();

                    long length = urlc.getContentLength();      // -1 if not available
                    InputStream is = urlc.getInputStream();

                    loadFromStream(new BufferedReader(new InputStreamReader(is)));
                    is.close();
                } catch (MalformedURLException ex) {
                    new SaviErrorMessage("Malformed URL, string could not be parsed.");
                } catch (IOException ex) { 
                    new SaviErrorMessage("Cannot open input file.");
                } catch (Exception ex) { 
                    new SaviErrorMessage(ex.toString());
                }
            }
        }
    }
            
    class BtRandomItems implements ActionListener {
        int init_vars = 21, init_claus = 90;
        
        public void actionPerformed(ActionEvent e) {
            running(false);
            SaviDialog sd = new SaviDialog("MaxSAT generator");
            TextField fieldVars, fieldClauses;
            sd.addInputRow("Variables count", fieldVars = new TextField("21", 3));
            sd.addInputRow("Clauses count", fieldClauses = new TextField("90", 4));
            fieldVars.setText(Integer.toString(init_vars));
            fieldClauses.setText(Integer.toString(init_claus));
            if (sd.showForm()) {
                int vars, claus;
                try {
                    vars = (int)Double.parseDouble(fieldVars.getText());
                    if ((vars < 4) || (vars > 500))
                        throw new Exception();
                } catch (Exception ex) {
                    new SaviErrorMessage("Variables count must be a numeric type between 5 and 500.");
                    return;
                }
                try {
                    claus = (int)Double.parseDouble(fieldClauses.getText());
                    if ((claus < 3) || (claus > 2500))
                        throw new Exception();
                } catch (Exception ex) {
                    new SaviErrorMessage("Clauses count must be a numeric type between 5 and 2500.");
                    return;
                }
                ((StateMaxSAT3)state).setFormula(new Formula(claus, vars));
                initAlg();
                updateDisplay();
                init_vars = vars;
                init_claus = claus;
            }
        }
    }
    
    class BrReduceFormula implements ActionListener {
        double init_percentage = 80;
        
        public void actionPerformed(ActionEvent e) {
            running(false);
            SaviDialog sd = new SaviDialog("MAXSAT - formula reducing");
            TextField fieldRed;
            sd.addInputRow("Reduce to [%]", fieldRed = new TextField("80", 2));
            fieldRed.setText(Double.toString(init_percentage));
            if (sd.showForm()) {
                double percentage;
                try {
                    percentage = Double.parseDouble(fieldRed.getText());
                    if ((percentage <= 0) || (percentage >= 100))
                        throw new NumberFormatException();

                    Formula f = formula.clone();
                    f.reduceNumberOfClauses(percentage / 100);
                    ((StateMaxSAT3)state).setFormula(f);
                    formula = f;
                    initAlg();
                    updateDisplay();
                    init_percentage = percentage;
                } catch (NumberFormatException ex) {
                    new SaviErrorMessage("Value must be a number between 1 and 100");
                } catch (WrongFormulaException ex) {
                    new SaviErrorMessage("Formula is too simple.");
                } catch (Exception ex) {
                    new SaviErrorMessage(ex.toString());
                }
            }
        }
    }

}
